// This file was generated by ecm_setup_version(): DO NOT EDIT!

#ifndef dismanwl_VERSION_H
#define dismanwl_VERSION_H

#define dismanwl_VERSION_STRING "0.527.0"
#define dismanwl_VERSION_MAJOR 0
#define dismanwl_VERSION_MINOR 527
#define dismanwl_VERSION_PATCH 0
#define dismanwl_VERSION ((0<<16)|(527<<8)|(0))

#endif
