
#ifndef DISMANWL_EXPORT_H
#define DISMANWL_EXPORT_H

#ifdef DISMANWL_STATIC_DEFINE
#  define DISMANWL_EXPORT
#  define DISMANWL_NO_EXPORT
#else
#  ifndef DISMANWL_EXPORT
#    ifdef disman_wllib_EXPORTS
        /* We are building this library */
#      define DISMANWL_EXPORT __attribute__((visibility("default")))
#    else
        /* We are using this library */
#      define DISMANWL_EXPORT __attribute__((visibility("default")))
#    endif
#  endif

#  ifndef DISMANWL_NO_EXPORT
#    define DISMANWL_NO_EXPORT __attribute__((visibility("hidden")))
#  endif
#endif

#ifndef DISMANWL_DEPRECATED
#  define DISMANWL_DEPRECATED __attribute__ ((__deprecated__))
#endif

#ifndef DISMANWL_DEPRECATED_EXPORT
#  define DISMANWL_DEPRECATED_EXPORT DISMANWL_EXPORT DISMANWL_DEPRECATED
#endif

#ifndef DISMANWL_DEPRECATED_NO_EXPORT
#  define DISMANWL_DEPRECATED_NO_EXPORT DISMANWL_NO_EXPORT DISMANWL_DEPRECATED
#endif

#if 0 /* DEFINE_NO_DEPRECATED */
#  ifndef DISMANWL_NO_DEPRECATED
#    define DISMANWL_NO_DEPRECATED
#  endif
#endif

#endif /* DISMANWL_EXPORT_H */
