// This file was generated by ecm_setup_version(): DO NOT EDIT!

#ifndef Disman_VERSION_H
#define Disman_VERSION_H

#define Disman_VERSION_STRING "0.527.0"
#define Disman_VERSION_MAJOR 0
#define Disman_VERSION_MINOR 527
#define Disman_VERSION_PATCH 0
#define Disman_VERSION ((0<<16)|(527<<8)|(0))

#endif
