
#ifndef DISMAN_EXPORT_H
#define DISMAN_EXPORT_H

#ifdef DISMAN_STATIC_DEFINE
#  define DISMAN_EXPORT
#  define DISMAN_NO_EXPORT
#else
#  ifndef DISMAN_EXPORT
#    ifdef disman_lib_EXPORTS
        /* We are building this library */
#      define DISMAN_EXPORT __attribute__((visibility("default")))
#    else
        /* We are using this library */
#      define DISMAN_EXPORT __attribute__((visibility("default")))
#    endif
#  endif

#  ifndef DISMAN_NO_EXPORT
#    define DISMAN_NO_EXPORT __attribute__((visibility("hidden")))
#  endif
#endif

#ifndef DISMAN_DEPRECATED
#  define DISMAN_DEPRECATED __attribute__ ((__deprecated__))
#endif

#ifndef DISMAN_DEPRECATED_EXPORT
#  define DISMAN_DEPRECATED_EXPORT DISMAN_EXPORT DISMAN_DEPRECATED
#endif

#ifndef DISMAN_DEPRECATED_NO_EXPORT
#  define DISMAN_DEPRECATED_NO_EXPORT DISMAN_NO_EXPORT DISMAN_DEPRECATED
#endif

#if 0 /* DEFINE_NO_DEPRECATED */
#  ifndef DISMAN_NO_DEPRECATED
#    define DISMAN_NO_DEPRECATED
#  endif
#endif

#endif /* DISMAN_EXPORT_H */
